import * as cdk from '@aws-cdk/core';
import { Construct } from 'constructs';
import { VirtualGateway, VirtualGatewayBaseProps } from './virtual-gateway';
import { VirtualNode, VirtualNodeBaseProps } from './virtual-node';
import { VirtualRouter, VirtualRouterBaseProps } from './virtual-router';
/**
 * A utility enum defined for the egressFilter type property, the default of DROP_ALL, allows traffic only to other resources inside the mesh, or API calls to amazon resources.
 *
 * @default DROP_ALL
 * @stability stable
 */
export declare enum MeshFilterType {
    /**
     * Allows all outbound traffic.
     *
     * @stability stable
     */
    ALLOW_ALL = "ALLOW_ALL",
    /**
     * Allows traffic only to other resources inside the mesh, or API calls to amazon resources.
     *
     * @stability stable
     */
    DROP_ALL = "DROP_ALL"
}
/**
 * Interface which all Mesh based classes MUST implement.
 *
 * @stability stable
 */
export interface IMesh extends cdk.IResource {
    /**
     * The name of the AppMesh mesh.
     *
     * @stability stable
     * @attribute true
     */
    readonly meshName: string;
    /**
     * The Amazon Resource Name (ARN) of the AppMesh mesh.
     *
     * @stability stable
     * @attribute true
     */
    readonly meshArn: string;
    /**
     * Creates a new VirtualRouter in this Mesh.
     *
     * Note that the Router is created in the same Stack that this Mesh belongs to,
     * which might be different than the current stack.
     *
     * @stability stable
     */
    addVirtualRouter(id: string, props?: VirtualRouterBaseProps): VirtualRouter;
    /**
     * Creates a new VirtualNode in this Mesh.
     *
     * Note that the Node is created in the same Stack that this Mesh belongs to,
     * which might be different than the current stack.
     *
     * @stability stable
     */
    addVirtualNode(id: string, props?: VirtualNodeBaseProps): VirtualNode;
    /**
     * Creates a new VirtualGateway in this Mesh.
     *
     * Note that the Gateway is created in the same Stack that this Mesh belongs to,
     * which might be different than the current stack.
     *
     * @stability stable
     */
    addVirtualGateway(id: string, props?: VirtualGatewayBaseProps): VirtualGateway;
}
/**
 * Represents a new or imported AppMesh mesh
 */
declare abstract class MeshBase extends cdk.Resource implements IMesh {
    /**
     * The name of the AppMesh mesh
     */
    abstract readonly meshName: string;
    /**
     * The Amazon Resource Name (ARN) of the AppMesh mesh
     */
    abstract readonly meshArn: string;
    /**
     * Adds a VirtualRouter to the Mesh with the given id and props.
     *
     * @stability stable
     */
    addVirtualRouter(id: string, props?: VirtualRouterBaseProps): VirtualRouter;
    /**
     * Adds a VirtualNode to the Mesh.
     *
     * @stability stable
     */
    addVirtualNode(id: string, props?: VirtualNodeBaseProps): VirtualNode;
    /**
     * Adds a VirtualGateway to the Mesh.
     *
     * @stability stable
     */
    addVirtualGateway(id: string, props?: VirtualGatewayBaseProps): VirtualGateway;
}
/**
 * The set of properties used when creating a Mesh.
 *
 * @stability stable
 */
export interface MeshProps {
    /**
     * The name of the Mesh being defined.
     *
     * @default - A name is automatically generated
     * @stability stable
     */
    readonly meshName?: string;
    /**
     * Egress filter to be applied to the Mesh.
     *
     * @default DROP_ALL
     * @stability stable
     */
    readonly egressFilter?: MeshFilterType;
}
/**
 * Define a new AppMesh mesh.
 *
 * @see https://docs.aws.amazon.com/app-mesh/latest/userguide/meshes.html
 * @stability stable
 */
export declare class Mesh extends MeshBase {
    /**
     * Import an existing mesh by arn.
     *
     * @stability stable
     */
    static fromMeshArn(scope: Construct, id: string, meshArn: string): IMesh;
    /**
     * Import an existing mesh by name.
     *
     * @stability stable
     */
    static fromMeshName(scope: Construct, id: string, meshName: string): IMesh;
    /**
     * The name of the AppMesh mesh.
     *
     * @stability stable
     */
    readonly meshName: string;
    /**
     * The Amazon Resource Name (ARN) of the AppMesh mesh.
     *
     * @stability stable
     */
    readonly meshArn: string;
    /**
     * @stability stable
     */
    constructor(scope: Construct, id: string, props?: MeshProps);
}
export {};
